<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Google Analytics GA4 Custom Form
 */

namespace Amasty\GA4CustomForm\Model\Event;

use Amasty\GA4\Model\ConfigProvider;
use Amasty\GA4\Model\Event\CommonEvent;
use Amasty\GA4Api\Model\Event\EventLoggerFactory;
use Amasty\GA4Api\Model\Event\EventsConfigPool;
use Amasty\GA4Api\Model\Event\Meta\IncrementIdStorage;
use Amasty\GA4CustomForm\Model\ConfigProvider as GA4CustomFormConfigProvider;
use Amasty\GA4CustomForm\Model\Event\Logger\CustomForm;
use Amasty\GA4CustomForm\Model\GtmMeta\FormVariables;

class FormClose extends CommonEvent
{
    public const NAME = 'amasty_customform_close';
    public const LOGGER_CLASS = CustomForm::class;

    /**
     * @var GA4CustomFormConfigProvider
     */
    private GA4CustomFormConfigProvider $ga4CustomFormConfigProvider;

    public function __construct(
        ConfigProvider $configProvider,
        EventLoggerFactory $eventLoggerFactory,
        IncrementIdStorage $incrementIdStorage,
        GA4CustomFormConfigProvider $ga4CustomFormConfigProvider,
        string $loggerClass = self::LOGGER_CLASS
    ) {
        parent::__construct($configProvider, $eventLoggerFactory, $incrementIdStorage, $loggerClass);

        $this->ga4CustomFormConfigProvider = $ga4CustomFormConfigProvider;
    }

    public function getMeta(?int $fingerprint = null): array
    {
        $fingerprint = $fingerprint ?? time();

        $meta = $this->buildCommonMeta('Amasty GA4 - ' . self::NAME, 'Amasty GA4 - CustomForm Close', $fingerprint);
        if (isset($meta['containerVersion']['tag'][0]['parameter'])) {
            foreach ($meta['containerVersion']['tag'][0]['parameter'] as &$param) {
                if ($param['key'] === 'eventParameters') {
                    $param['list'] = $this->buildEventParameter(
                        FormVariables::FORM_ID_CODE,
                        FormVariables::FORM_ID_NAME
                    );
                }
            }
        }

        return $meta;
    }

    public function isAvailable(): bool
    {
        return $this->ga4CustomFormConfigProvider->isEnableForm();
    }

    public function getUsageType(): string
    {
        return EventsConfigPool::USAGE_HEADER_DATALAYER;
    }
}
