<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Google Analytics GA4 Custom Form
 */

namespace Amasty\GA4CustomForm\Model\GtmMeta;

use Amasty\GA4\Model\GtmMeta\Utils\VariablesProvider;
use Amasty\GA4Api\Model\GtmMeta\GtmMetaInterface;
use Amasty\GA4CustomForm\Model\ConfigProvider;
use Magento\Framework\Stdlib\ArrayManager;

class FormVariables implements GtmMetaInterface
{
    public const VARIABLES_PATH = 'containerVersion/variable';

    public const FORM_ID_CODE = 'form_id';
    public const FORM_ID_NAME = 'Amasty - GA4 - Custom Form ID';

    /**
     * @var ArrayManager
     */
    private ArrayManager $arrayManager;

    /**
     * @var ConfigProvider
     */
    private ConfigProvider $configProvider;

    /**
     * @var VariablesProvider
     */
    private VariablesProvider $variablesProvider;

    public function __construct(
        ArrayManager $arrayManager,
        ConfigProvider $configProvider,
        VariablesProvider $variablesProvider
    ) {
        $this->arrayManager = $arrayManager;
        $this->configProvider = $configProvider;
        $this->variablesProvider = $variablesProvider;
    }

    public function collect(int $fingerprint = null, array &$data = []): void
    {
        $variables = $this->arrayManager->get(self::VARIABLES_PATH, $data);

        if (null === $variables) {
            return;
        }

        $variables[] = $this->getFormVariables($fingerprint ??= time());
        $data = $this->arrayManager->set(self::VARIABLES_PATH, $data, $variables);
    }

    private function getFormVariables(int $fingerprint): array
    {
        return $this->variablesProvider->prepareBasicVariable(
            self::FORM_ID_NAME,
            self::FORM_ID_CODE,
            $this->configProvider->getAccountId(),
            $this->configProvider->getContainerId(),
            $fingerprint
        );
    }
}
